!SFX_LIC Copyright 1994-2014 CNRS, Meteo-France and Universite Paul Sabatier
!SFX_LIC This is part of the SURFEX software governed by the CeCILL-C licence
!SFX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt  
!SFX_LIC for details. version 1.
!     ###############################################################################
SUBROUTINE COUPLING_ISBA_n (DTCO, UG, U, USS, NAG, CHI, NCHI, MGN, MSF,  DTI, ID, NGB,  &
                            GB, ISS, NISS, IG, NIG, IO, S, K, NK, NP, NPE, AT, NDST,    &
                            DST,SLT, HPROGRAM, HCOUPLING, PTSTEP, KYEAR, KMONTH, KDAY,  &
                            PTIME, KI, KSV, KSW, PTSUN, PZENITH, PZENITH2, PAZIM,       &
                            PZREF,PUREF, PZS, PU, PV, PQA, PTA, PRHOA, PSV, PCO2,       &
                            PIMPWET, PIMPDRY, HSV, PRAIN, PSNOW, PLW,                   &
                            PDIR_SW, PSCA_SW, PSW_BANDS, PPS, PPA, PSFTQ, PSFTH, PSFTS, &
                            PSFCO2, PSFU, PSFV, PTRAD, PDIR_ALB, PSCA_ALB, PEMIS,       &
                            PTSURF, PZ0, PZ0H, PQSURF, PPEW_A_COEF, PPEW_B_COEF,        &
                            PPET_A_COEF, PPEQ_A_COEF, PPET_B_COEF, PPEQ_B_COEF, HTEST   )  
!     ###############################################################################
!
!!****  *COUPLING_ISBA_n * - Driver for ISBA time step   
!!
!!    PURPOSE
!!    -------
!
!!**  METHOD
!!    ------
!!
!! First, all actions dependant on each patch is donbe independantly
!!     (loop on patches)
!! Second, actions common to all patches (e.g. prescription of new vegetation)
!! Third, energy fluxes are averaged
!!
!! Nota that chemical fluxes are also treated.
!!
!!    REFERENCE
!!    ---------
!!      
!!
!!    AUTHOR
!!    ------
!!     V. Masson 
!!
!!    MODIFICATIONS
!!    -------------
!!      Original    01/2004
!!      P Le Moigne 11/2004 add new diagnostics for isba
!!      A.Bogatchev 09/2005 EBA snow option
!!      P Le Moigne 09/2005 AGS modifs of L. Jarlan
!!      P Le Moigne 02/2006 z0h with snow
!!      P.Le Moigne 06/2006 seeding and irrigation
!!      B. Decharme   2008  reset the subgrid topographic effect on the forcing
!!                          PSNV allways <= PSNG
!!                          News diag
!!                          Flooding scheme and allows TRIP variables coupling
!!      A.L. Gibelin 04/2009 : Add respiration diagnostics
!!      A.L. Gibelin 04/2009 : BIOMASS and RESP_BIOMASS arrays 
!!      A.L. Gibelin 04/2009 : TAU_WOOD for NCB option 
!!      A.L. Gibelin 05/2009 : Add carbon spinup
!!      A.L. Gibelin 06/2009 : Soil carbon variables for CNT option
!!      A.L. Gibelin 07/2009 : Suppress RDK and transform GPP as a diagnostic
!!      A.L. Gibelin 07/2009 : Suppress PPST and PPSTF as outputs
!!        S.Lafont   01/2011 : add PTSTEP as arg of diag_misc
!!       B.Decharme  09/2012 : Bug in hydro_glacier calculation with ES or Crocus
!!                             New wind implicitation
!!                             New soil carbon spinup and diag
!!                             Isba budget
!!      F. Bouttier  01/2013 : Apply random perturbations for ensembles
!!      B. Decharme  04/2013 new coupling variables
!!                           Subsurface runoff if SGH (DIF option only)
!!                   07/2013 Surface / Water table depth coupling
!!      P Samuelsson 10/2014 : MEB
!!      P. LeMoigne  12/2014 EBA scheme update
!!      R. Seferian  05/2015 : Add coupling fiels to vegetation_evol call
!!      F. Tuzet     06/2016 : Add of a new dimension for impurity: The type of impurity
!!      P. Tulet     06/2016 : call coupling_megan add RN leaves for MEGAN
!!      A. Druel     02/2019 : Adapt the code to be compatible with new irrigation
!!                             and avoid agricultural pratices for trees / shrubs
!!      J. Pianezzej 02/2019 : correction for use of MEGAN
!!      J. Colin       08/16 : Add the snow and soil moisture nudging
!!      R. Séférian    11/16 : Fire scheme
!!      R. Séférian    11/16 : Implement carbon cycle coupling (Earth system model)
!!      B. Decharme  02/2019 : Explicit soil carbon and gas schemes
!!                             Delete wood carbon spinup
!!                             ISBA-CC diag in DIAG_CC_CUMUL_ISBA and AVERAGE_DIAG_EVAP_ISBA_CC
!!-------------------------------------------------------------------
!
USE MODD_PREP_SNOW, ONLY    : NIMPUR

#ifdef SFX_OL
USE MODN_IO_OFFLINE, ONLY   : LFORCIMP
#endif
!
USE MODD_DATA_COVER_n,    ONLY : DATA_COVER_t
USE MODD_SURF_ATM_GRID_n, ONLY : SURF_ATM_GRID_t
USE MODD_SURF_ATM_n,      ONLY : SURF_ATM_t
!
USE MODD_AGRI_n,          ONLY : AGRI_NP_t
USE MODD_CH_ISBA_n,       ONLY : CH_ISBA_t, CH_ISBA_NP_t
USE MODD_MEGAN_n,         ONLY : MEGAN_t
USE MODD_MEGAN_SURF_FIELDS_n, ONLY : MEGAN_SURF_FIELDS_t
USE MODD_DATA_ISBA_n,     ONLY : DATA_ISBA_t
USE MODD_SURFEX_n,        ONLY : ISBA_DIAG_t
USE MODD_GR_BIOG_n,       ONLY : GR_BIOG_t, GR_BIOG_NP_t
USE MODD_SSO_n,           ONLY : SSO_t, SSO_NP_t
USE MODD_SFX_GRID_n,      ONLY : GRID_t, GRID_NP_t
USE MODD_ISBA_OPTIONS_n,  ONLY : ISBA_OPTIONS_t
USE MODD_ISBA_n,          ONLY : ISBA_S_t, ISBA_K_t, ISBA_P_t, ISBA_PE_t, &
                                 ISBA_NK_t, ISBA_NP_t, ISBA_NPE_t
!
USE MODD_DST_n,           ONLY : DST_NP_t, DST_t
USE MODD_CHS_AEROSOL,     ONLY : XEMISSIGI, XEMISSIGJ,&
                                 XEMISRADIUSI, XEMISRADIUSJ, CRGUNIT
USE MODD_SLT_n,           ONLY : SLT_t
!
USE MODD_REPROD_OPER,     ONLY : CIMPLICIT_WIND
!
USE MODD_CSTS,            ONLY : XRD, XP00, XCPD, XPI, XAVOGADRO, XMD, XDAY
USE MODD_CO2V_PAR,        ONLY : XMCO2, XMCO2, XMC, XGTOKG
!
USE MODD_SURF_PAR,        ONLY : XUNDEF
USE MODD_TYPE_DATE_SURF
!
USE MODD_SURF_ATM,        ONLY : LNOSOF, LSLOPE
!
USE MODD_DST_SURF
USE MODD_SLT_SURF
USE MODE_DSLT_SURF
USE MODE_MEB
!
USE MODD_DATA_COVER_PAR, ONLY : NVT_TEBD, NVT_BONE, NVT_TRBE, NVT_TRBD, &
                                NVT_TEBE, NVT_TENE, NVT_BOBD, NVT_BOND, &
                                NVT_SHRB, NVT_FLTR, NVEGTYPE
!
USE MODD_AGRI,            ONLY : LAGRIP, LIRRIGMODE, LMULTI_SEASON, NVEG_IRR
USE MODD_DEEPSOIL,        ONLY : LDEEPSOIL
!
#ifdef TOPD
USE MODD_COUPLING_TOPD,   ONLY : LCOUPL_TOPD, NMASKT_PATCH
#endif
!
USE MODD_SURF_ATM_TURB_n, ONLY : SURF_ATM_TURB_t
!
USE MODI_ADD_FORECAST_TO_DATE_SURF
USE MODI_Z0EFF
USE MODI_ISBA
USE MODI_AVERAGE_FLUX
USE MODI_AVERAGE_PHY
USE MODI_AVERAGE_RAD
USE MODI_AVERAGE_DIAG_ISBA_n
USE MODI_VEGETATION_EVOL
USE MODI_VEGETATION_UPDATE
USE MODI_CARBON_EVOL
USE MODI_CARBON_EVOL_DIF
USE MODI_SUBSCALE_Z0EFF
USE MODI_SOIL_ALBEDO
USE MODI_ALBEDO
USE MODI_DIAG_INLINE_ISBA_n
USE MODI_DIAG_EVAP_CUMUL_ISBA_n
USE MODI_DIAG_CC_CUMUL_ISBA_n
USE MODI_DIAG_MISC_ISBA_n
USE MODI_REPROJ_DIAG_ISBA_n
!
USE MODI_UPDATE_RAD_ISBA_n
USE MODI_DEEPSOIL_UPDATE
USE MODI_ISBA_SGH_UPDATE
USE MODI_ISBA_FLOOD_PROPERTIES
USE MODI_DIAG_CPL_ESM_ISBA
USE MODI_HYDRO_GLACIER
USE MODI_ISBA_ALBEDO
USE MODI_CARBON_SPINUP
USE MODI_CH_AER_DEP
USE MODI_ABOR1_SFX
USE MODI_AVERAGE_DIAG_EVAP_ISBA_n
USE MODI_AVERAGE_DIAG_CC_ISBA_n
USE MODI_AVERAGE_DIAG_MISC_ISBA_n
USE MODI_CH_BVOCEM_n
USE MODI_SOILEMISNO_n
USE MODI_CH_DEP_ISBA
USE MODI_DSLT_DEP
USE MODI_COUPLING_DST_n
USE MODI_COUPLING_SURF_TOPD
USE MODI_ISBA_BUDGET_INIT
USE MODI_ISBA_BUDGET
USE MODI_UNPACK_DIAG_CPL_ESM_ISBA
USE MODD_SYTRON_PAR, ONLY : NTAB_SYT
USE MODI_VEGTYPE_TO_PATCH_IRRIG
USE MODI_COUPLING_MEGAN_n
USE MODI_FIRE_DISTURBANCE
USE MODI_NUDGING_ISBA_SNOW
USE MODI_NUDGING_ISBA_SOIL
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE
!
!*      0.1    declarations of arguments
!
TYPE(DATA_COVER_t),    INTENT(INOUT) :: DTCO
TYPE(SURF_ATM_GRID_t), INTENT(INOUT) :: UG
TYPE(SURF_ATM_t),      INTENT(INOUT) :: U
TYPE(SSO_t),           INTENT(INOUT) :: USS
!
TYPE(AGRI_NP_t),       INTENT(INOUT) :: NAG
TYPE(CH_ISBA_t),       INTENT(INOUT) :: CHI
TYPE(CH_ISBA_NP_t),    INTENT(INOUT) :: NCHI
TYPE(MEGAN_t),         INTENT(INOUT) :: MGN
TYPE(MEGAN_SURF_FIELDS_t), INTENT(INOUT) :: MSF
TYPE(DATA_ISBA_t),     INTENT(INOUT) :: DTI
TYPE(ISBA_DIAG_t),     INTENT(INOUT) :: ID
TYPE(GR_BIOG_NP_t),    INTENT(INOUT) :: NGB
TYPE(GR_BIOG_t),       INTENT(INOUT) :: GB
TYPE(SSO_t),           INTENT(INOUT) :: ISS 
TYPE(SSO_NP_t),        INTENT(INOUT) :: NISS
TYPE(GRID_t),          INTENT(INOUT) :: IG
TYPE(GRID_NP_t),       INTENT(INOUT) :: NIG
TYPE(ISBA_OPTIONS_t),  INTENT(INOUT) :: IO
TYPE(ISBA_S_t),        INTENT(INOUT) :: S
TYPE(ISBA_K_t),        INTENT(INOUT) :: K
TYPE(ISBA_NK_t),       INTENT(INOUT) :: NK
TYPE(ISBA_NP_t),       INTENT(INOUT) :: NP
TYPE(ISBA_NPE_t),      INTENT(INOUT) :: NPE
TYPE(SURF_ATM_TURB_t), INTENT(IN)    :: AT         ! atmospheric turbulence parameters
!
TYPE(DST_t), INTENT(INOUT) :: DST
TYPE(DST_NP_t), INTENT(INOUT) :: NDST
TYPE(SLT_t), INTENT(INOUT) :: SLT
!
CHARACTER(LEN=6),    INTENT(IN)  :: HPROGRAM  ! program calling surf. schemes
CHARACTER(LEN=1),    INTENT(IN)  :: HCOUPLING ! type of coupling
                                              ! 'E' : explicit
                                              ! 'I' : implicit
INTEGER,             INTENT(IN)  :: KYEAR     ! current year (UTC)
INTEGER,             INTENT(IN)  :: KMONTH    ! current month (UTC)
INTEGER,             INTENT(IN)  :: KDAY      ! current day (UTC)
REAL,                INTENT(IN)  :: PTIME     ! current time since midnight (UTC, s)
INTEGER,             INTENT(IN)  :: KI        ! number of points
INTEGER,             INTENT(IN)  :: KSV       ! number of scalars
INTEGER,             INTENT(IN)  :: KSW       ! number of short-wave spectral bands
REAL, DIMENSION(KI), INTENT(IN)  :: PTSUN     ! solar time                    (s from midnight)
REAL,                INTENT(IN)  :: PTSTEP    ! atmospheric time-step                 (s)
REAL, DIMENSION(KI), INTENT(IN)  :: PZREF     ! height of T,q forcing                 (m)
REAL, DIMENSION(KI), INTENT(IN)  :: PUREF     ! height of wind forcing                (m)
!
REAL, DIMENSION(KI), INTENT(IN)  :: PTA       ! air temperature forcing               (K)
REAL, DIMENSION(KI), INTENT(IN)  :: PQA       ! air humidity forcing                  (kg/m3)
REAL, DIMENSION(KI), INTENT(IN)  :: PRHOA     ! air density                           (kg/m3)
REAL, DIMENSION(KI,KSV),INTENT(IN) :: PSV     ! scalar variables
!                                             ! chemistry:   first char. in HSV: '#'  (molecule/m3)
!   
 CHARACTER(LEN=6), DIMENSION(KSV),INTENT(IN):: HSV  ! name of all scalar variables!
REAL, DIMENSION(KI), INTENT(IN)  :: PU        ! zonal wind                            (m/s)
REAL, DIMENSION(KI), INTENT(IN)  :: PV        ! meridian wind                         (m/s)
REAL, DIMENSION(KI,KSW),INTENT(IN) :: PDIR_SW ! direct  solar radiation (on horizontal surf.)
!                                             !                                       (W/m2)
REAL, DIMENSION(KI,KSW),INTENT(IN) :: PSCA_SW ! diffuse solar radiation (on horizontal surf.)
!                                             !                                       (W/m2)
REAL, DIMENSION(KSW),INTENT(IN)  :: PSW_BANDS ! mean wavelength of each shortwave band (m)
REAL, DIMENSION(KI), INTENT(IN)  :: PZENITH   ! zenithal angle at t  (radian from the vertical)
REAL, DIMENSION(KI), INTENT(IN)  :: PZENITH2  ! zenithal angle at t+1(radian from the vertical)
REAL, DIMENSION(KI), INTENT(IN)  :: PAZIM     ! azimuthal angle      (radian from North, clockwise)
REAL, DIMENSION(KI), INTENT(IN)  :: PLW       ! longwave radiation (on horizontal surf.)
!                                             !                                       (W/m2)
REAL, DIMENSION(KI), INTENT(IN)  :: PPS       ! pressure at atmospheric model surface (Pa)
REAL, DIMENSION(KI), INTENT(IN)  :: PPA       ! pressure at forcing level             (Pa)
REAL, DIMENSION(KI), INTENT(IN)  :: PZS       ! atmospheric model orography           (m)
REAL, DIMENSION(KI), INTENT(IN)  :: PCO2      ! CO2 concentration in the air          (kg_CO2/m3)
REAL, DIMENSION(KI,NIMPUR), INTENT(IN) :: PIMPWET ! Wet impur deposition
REAL, DIMENSION(KI,NIMPUR), INTENT(IN) :: PIMPDRY ! Dry impur deposition
REAL, DIMENSION(KI), INTENT(IN)  :: PSNOW     ! snow precipitation                    (kg/m2/s)
REAL, DIMENSION(KI), INTENT(IN)  :: PRAIN     ! liquid precipitation                  (kg/m2/s)
!
!
REAL, DIMENSION(KI), INTENT(OUT) :: PSFTH     ! flux of heat                          (W/m2)
REAL, DIMENSION(KI), INTENT(OUT) :: PSFTQ     ! flux of water vapor                   (kg/m2/s)
REAL, DIMENSION(KI), INTENT(OUT) :: PSFU      ! zonal momentum flux                   (Pa)
REAL, DIMENSION(KI), INTENT(OUT) :: PSFV      ! meridian momentum flux                (Pa)
REAL, DIMENSION(KI), INTENT(OUT) :: PSFCO2    ! flux of CO2 positive toward the atmosphere (m/s*kg_CO2/kg_air)
REAL, DIMENSION(KI,KSV),INTENT(OUT):: PSFTS   ! flux of scalar var.                   (kg/m2/s)
!
REAL, DIMENSION(KI), INTENT(OUT) :: PTRAD     ! radiative temperature                 (K)
REAL, DIMENSION(KI,KSW),INTENT(OUT):: PDIR_ALB! direct albedo for each spectral band  (-)
REAL, DIMENSION(KI,KSW),INTENT(OUT):: PSCA_ALB! diffuse albedo for each spectral band (-)
REAL, DIMENSION(KI), INTENT(OUT) :: PEMIS     ! emissivity                            (-)
!
REAL, DIMENSION(KI), INTENT(OUT) :: PTSURF    ! surface effective temperature         (K)
REAL, DIMENSION(KI), INTENT(OUT) :: PZ0       ! roughness length for momentum         (m)
REAL, DIMENSION(KI), INTENT(OUT) :: PZ0H      ! roughness length for heat             (m)
REAL, DIMENSION(KI), INTENT(OUT) :: PQSURF    ! specific humidity at surface          (kg/kg)
!
REAL, DIMENSION(KI), INTENT(IN) :: PPEW_A_COEF! implicit coefficients
REAL, DIMENSION(KI), INTENT(IN) :: PPEW_B_COEF! needed if HCOUPLING='I'
REAL, DIMENSION(KI), INTENT(IN) :: PPET_A_COEF
REAL, DIMENSION(KI), INTENT(IN) :: PPEQ_A_COEF
REAL, DIMENSION(KI), INTENT(IN) :: PPET_B_COEF
REAL, DIMENSION(KI), INTENT(IN) :: PPEQ_B_COEF
CHARACTER(LEN=2),    INTENT(IN) :: HTEST ! must be equal to 'OK'

!
!
!*      0.2    declarations of local variables
!
!* forcing variables
!
TYPE(ISBA_K_t),  POINTER :: KK
TYPE(ISBA_P_t),  POINTER :: PK
TYPE(ISBA_PE_t), POINTER :: PEK
TYPE(SSO_t),     POINTER :: ISSK
!
REAL, DIMENSION(KI)     :: ZWIND    ! lowest atmospheric level wind speed           (m/s)
REAL, DIMENSION(KI)     :: ZDIR     ! wind direction                        (rad from N clockwise)
REAL, DIMENSION(KI)     :: ZEXNA    ! Exner function at lowest atmospheric level    (-)
REAL, DIMENSION(KI)     :: ZEXNS    ! Exner function at surface                     (-)
REAL, DIMENSION(KI)     :: ZALFA    ! Wind direction                                (-)
REAL, DIMENSION(KI)     :: ZQA      ! specific humidity                             (kg/kg)
REAL, DIMENSION(KI)     :: ZCO2     ! CO2 concentration                             (kg/kg)
REAL, DIMENSION(KI)     :: ZPEQ_A_COEF ! specific humidity implicit
REAL, DIMENSION(KI)     :: ZPEQ_B_COEF ! coefficients (hum. in kg/kg)
!
! Patch outputs:
!
REAL, DIMENSION(KI,IO%NPATCH) :: ZSFTH_TILE     ! surface heat flux (W/m2)
REAL, DIMENSION(KI,IO%NPATCH) :: ZSFTQ_TILE     ! surface vapor flux (kg/m2/s)
REAL, DIMENSION(KI,IO%NPATCH) :: ZSFCO2_TILE    ! surface CO2 flux positive toward the atmosphere (m/s*kg_CO2/kg_air)
REAL, DIMENSION(KI,IO%NPATCH) :: ZSFU_TILE      ! zonal momentum flux
REAL, DIMENSION(KI,IO%NPATCH) :: ZSFV_TILE      ! meridian momentum flux
REAL, DIMENSION(KI,IO%NPATCH) :: ZTRAD_TILE     ! radiative surface temperature
REAL, DIMENSION(KI,IO%NPATCH) :: ZEMIS_TILE     ! emissivity
REAL, DIMENSION(KI,IO%NPATCH) :: ZTSURF_TILE    ! surface effective temperature
REAL, DIMENSION(KI,IO%NPATCH) :: ZZ0_TILE       ! roughness length for momentum
REAL, DIMENSION(KI,IO%NPATCH) :: ZZ0H_TILE      ! roughness length for heat
REAL, DIMENSION(KI,IO%NPATCH) :: ZQSURF_TILE    ! specific humidity at surface
REAL, DIMENSION(KI,KSW,IO%NPATCH) :: ZDIR_ALB_TILE  ! direct albedo
REAL, DIMENSION(KI,KSW,IO%NPATCH) :: ZSCA_ALB_TILE  ! diffuse albedo
REAL, DIMENSION(KI,KSV,IO%NPATCH) :: ZSFTS_TILE     ! scalar surface flux
!
REAL, DIMENSION(KI, IO%NPATCH) :: ZCPL_DRAIN     ! For the coupling with TRIP
REAL, DIMENSION(KI, IO%NPATCH) :: ZCPL_RUNOFF    ! For the coupling with TRIP
REAL, DIMENSION(KI, IO%NPATCH) :: ZCPL_EFLOOD    ! For the coupling with TRIP
REAL, DIMENSION(KI, IO%NPATCH) :: ZCPL_PFLOOD    ! For the coupling with TRIP
REAL, DIMENSION(KI, IO%NPATCH) :: ZCPL_IFLOOD    ! For the coupling with TRIP
REAL, DIMENSION(KI, IO%NPATCH) :: ZCPL_ICEFLUX   ! For the coupling with TRIP
REAL, DIMENSION(KI, IO%NPATCH) :: ZCPL_DOCFLUX   ! For the coupling with TRIP
!
! for chemical computations
!
REAL, DIMENSION(KI, IO%NPATCH) :: ZSW_FORBIO
!
REAL, DIMENSION(KI) :: ZRNSHADE
REAL, DIMENSION(KI) :: ZRNSUNLIT
!
REAL                       :: ZCONVERTFACM0_SLT, ZCONVERTFACM0_DST
REAL                       :: ZCONVERTFACM3_SLT, ZCONVERTFACM3_DST
REAL                       :: ZCONVERTFACM6_SLT, ZCONVERTFACM6_DST
REAL                       :: ZEMISRADIUSI, ZEMISRADIUSJ
!
! for blowing snow scheme
!
REAL, DIMENSION(KI,CHI%SVI%N2DSNWEQ) :: ZP_BLOWSNW_FLUX      ! blowing snow fluxes
REAL, DIMENSION(KI,CHI%SVI%NSNWEQ)   :: ZP_BLOWSNW_CONC      ! blowing snow concentration
!
! dimensions and loop counters
!
INTEGER :: ISWB   ! number of spectral shortwave bands
INTEGER :: JSWB   ! loop on number of spectral shortwave bands
INTEGER :: JP, JVEG ! loop on patches / on vegtypes
INTEGER :: JSV, IDST, IMOMENT, IMASK, JI, JK, JV, II
INTEGER :: JMODE, JSV_IDX
LOGICAL :: LIS_TREE
!
! logical units
!
INTEGER :: JJ, IBEG, IEND, ISIZE
LOGICAL :: GUPDATED  ! return F if VEGETATION_UPDATE has reset fields
!
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! --------------------------------------------------------------------------------------
IF (LHOOK) CALL DR_HOOK('COUPLING_ISBA_N',0,ZHOOK_HANDLE)
!
IF (HTEST/='OK') THEN
  CALL ABOR1_SFX('COUPLING_ISBAN: FATAL ERROR DURING ARGUMENT TRANSFER')
END IF
! --------------------------------------------------------------------------------------
!
!*      1.     Initializations
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Allocations:
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
ZSFTH_TILE   (:,:)   = XUNDEF
ZSFTQ_TILE   (:,:)   = XUNDEF
ZSFCO2_TILE  (:,:)   = XUNDEF
ZSFU_TILE    (:,:)   = XUNDEF
ZSFV_TILE    (:,:)   = XUNDEF
ZTRAD_TILE   (:,:)   = XUNDEF
ZEMIS_TILE   (:,:)   = XUNDEF
ZDIR_ALB_TILE(:,:,:) = XUNDEF
ZSCA_ALB_TILE(:,:,:) = XUNDEF
ZTSURF_TILE  (:,:)   = XUNDEF
ZZ0_TILE     (:,:)   = XUNDEF
ZZ0H_TILE    (:,:)   = XUNDEF
ZQSURF_TILE  (:,:)   = XUNDEF
!
ZSFTS_TILE(:,:,:) = 0.
!
ZCPL_DRAIN(:,:)   = 0.0
ZCPL_RUNOFF(:,:)  = 0.0
ZCPL_EFLOOD(:,:)  = 0.0
ZCPL_PFLOOD(:,:)  = 0.0
ZCPL_IFLOOD(:,:)  = 0.0
ZCPL_ICEFLUX(:,:) = 0.0
ZCPL_DOCFLUX(:,:) = 0.0
!
ZSW_FORBIO(:,:)   =  XUNDEF
!
ZRNSHADE(:)       = 0.0
ZRNSUNLIT(:)      = 0.0
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Forcing Modifications:
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
ZDIR=0.
!
DO JJ=1,SIZE(PQA) 
! specific humidity (conversion from kg/m3 to kg/kg)
!
  ZQA(JJ) = PQA(JJ) / PRHOA(JJ)
  ZPEQ_A_COEF(JJ) = PPEQ_A_COEF(JJ) / PRHOA(JJ)
  ZPEQ_B_COEF(JJ) = PPEQ_B_COEF(JJ) / PRHOA(JJ)
!
  ZCO2(JJ) = PCO2(JJ) / PRHOA(JJ)
!
! Other forcing variables depending on incoming forcing (argument list)JJ
!
! Optimization : X**n = EXP(n*LOG(X))
  ZEXNS(JJ)   = EXP((XRD/XCPD)*LOG(PPS(JJ)/XP00))
  ZEXNA(JJ)   = EXP((XRD/XCPD)*LOG(PPA(JJ)/XP00))
!
!* wind strength
!
  ZWIND(JJ) = SQRT(PU(JJ)**2+PV(JJ)**2)
!
!* wind direction
!
  IF (ZWIND(JJ)>0.)  ZDIR(JJ)=ATAN2(PU(JJ),PV(JJ))
!
!* angle between z0eff J axis and wind direction (rad., clockwise)
!
  ZALFA(JJ) = ZDIR(JJ) - ISS%XZ0EFFJPDIR(JJ) * XPI/180.

  IF (ZALFA(JJ)<-XPI) ZALFA(JJ) = ZALFA(JJ) + 2.*XPI
  IF (ZALFA(JJ)>=XPI) ZALFA(JJ) = ZALFA(JJ) - 2.*XPI
!
ENDDO
!
!* number of shortwave spectral bands
!
ISWB = KSW
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!* Soil moisture nudging
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
IF (IO%CNUDG_WG/='DEF') THEN   
   CALL NUDGING_ISBA_SOIL(IO, K, NK, NP, NPE, ID%DM, ID%NDM, S%TTIME%TDATE%YEAR, KMONTH, KDAY, PTSTEP)
ENDIF
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!* Actualization of the SGH variable (Fmu, Fsat)
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
CALL ISBA_SGH_UPDATE(IG, IO, S, K, NK, NP, NPE, PRAIN )
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!* Actualization of deep soil characteristics
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
IF (LDEEPSOIL) THEN
  DO JP = 1,IO%NPATCH
    KK => NK%AL(JP)
    CALL DEEPSOIL_UPDATE(KK%XTDEEP, KK%XGAMMAT, S%TTIME%TDATE%MONTH)
  ENDDO
ENDIF
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!* Actualization of soil and wood carbon spinup
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
IF(IO%LSPINUPCARBS)THEN
  CALL CARBON_SPINUP( S%TTIME, IO )
ELSE
  IO%NSPINS = 1
ENDIF
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Time evolution
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
S%TTIME%TIME = S%TTIME%TIME + PTSTEP
CALL ADD_FORECAST_TO_DATE_SURF(S%TTIME%TDATE%YEAR,S%TTIME%TDATE%MONTH,S%TTIME%TDATE%DAY,S%TTIME%TIME)
!
! --------------------------------------------------------------------------------------
!
!*      2.     Physical evolution
!
! --------------------------------------------------------------------------------------
! Patch Dependent Calculations
! --------------------------------------------------------------------------------------
!
PATCH_LOOP: DO JP=1,IO%NPATCH
  !
  IF (NP%AL(JP)%NSIZE_P == 0 ) CYCLE
  !
  ! Pack dummy arguments for each patch:
  !
#ifdef TOPD
    IF (LCOUPL_TOPD) THEN
      NMASKT_PATCH(:) = 0
      NMASKT_PATCH(1:NP%AL(JP)%NSIZE_P) = NP%AL(JP)%NR_P(:)
    ENDIF
#endif
  !
  ! Check if is tree or not to OAGRIP
  LIS_TREE = .FALSE.
  DO  JVEG=1, NVEG_IRR+NVEGTYPE
    CALL VEGTYPE_TO_PATCH_IRRIG(JVEG,IO%NPATCH,DTI%NPAR_VEG_IRR_USE,JK)
    IF ( JK /= JP ) CYCLE
    JV=JVEG
    IF ( JVEG > NVEGTYPE ) JV = DTI%NPAR_VEG_IRR_USE(JVEG-NVEGTYPE)
    IF ( ANY((/NVT_TEBD, NVT_BONE, NVT_TRBE, NVT_TRBD, NVT_TEBE, &
                NVT_TENE, NVT_BOBD, NVT_BOND, NVT_SHRB, NVT_FLTR/)==JV) ) LIS_TREE = .TRUE.
  ENDDO
  !
  CALL TREAT_PATCH(NK%AL(JP), NP%AL(JP), NPE%AL(JP), NISS%AL(JP), NAG%AL(JP), &
                   NIG%AL(JP), NCHI%AL(JP), NDST%AL(JP), DST, SLT, ID%ND%AL(JP), ID%NDC%AL(JP), &
                   ID%NDE%AL(JP), ID%NDEC%AL(JP), ID%NDM%AL(JP), NGB%AL(JP), LIS_TREE)
  !
ENDDO PATCH_LOOP
!
! --------------------------------------------------------------------------------------
! SFX - RRM coupling update if used :
! --------------------------------------------------------------------------------------
!
IF(IO%LCPL_RRM)THEN
  CALL DIAG_CPL_ESM_ISBA(IG, IO, S, NK, NP, PTSTEP, ZCPL_DRAIN, ZCPL_RUNOFF, &
                         ZCPL_EFLOOD, ZCPL_PFLOOD, ZCPL_IFLOOD, ZCPL_ICEFLUX,&
                         ZCPL_DOCFLUX                                        )  
ENDIF
!
! --------------------------------------------------------------------------------------
! Vegetation update (in case of non-interactive vegetation):
! Or
! Vegetation albedo only update (in case of interactive vegetation):
! --------------------------------------------------------------------------------------
!
GUPDATED=.FALSE.
!
IF (IO%LVEGUPD) THEN
   DO JP = 1,IO%NPATCH
      CALL VEGETATION_UPDATE(DTCO, DTI, IG%NDIM, IO, NK%AL(JP), NP%AL(JP), NPE%AL(JP), &
                             KMONTH, KDAY, JP, PTSTEP, S%TTIME, S%XCOVER, S%LCOVER,    &
                             LAGRIP, U%LECOSG, LIRRIGMODE, 'NAT', NISS%AL(JP), GUPDATED)  
   ENDDO
ENDIF
!
IF(IO%LPERTSURF.AND.GUPDATED) THEN
  !
  DO JP = 1,IO%NPATCH
     !
     PK => NP%AL(JP)
     PEK => NPE%AL(JP)
     ISSK => NISS%AL(JP)
     DO JI = 1,PK%NSIZE_P
        !
        IMASK = PK%NR_P(JI)
        !
        ! random perturbation for ensembles:
        ! reset these fields to their original values, as in compute_isba_parameters
        !
        PEK%XVEG(JI) = S%XPERTVEG(IMASK)
        PEK%XLAI(JI) = S%XPERTLAI(IMASK)
        PEK%XCV (JI) = S%XPERTCV (IMASK)
        !
        ! reapply original perturbation patterns
        !
        IF(PEK%XALBNIR(JI)/=XUNDEF)   PEK%XALBNIR(JI)   = PEK%XALBNIR(JI) *( 1.+ S%XPERTALB(IMASK) )
        IF(PEK%XALBVIS(JI)/=XUNDEF)   PEK%XALBVIS(JI)   = PEK%XALBVIS(JI) *( 1.+ S%XPERTALB(IMASK) )
        IF(PEK%XALBUV(JI)/=XUNDEF)    PEK%XALBUV (JI)   = PEK%XALBUV (JI) *( 1.+ S%XPERTALB(IMASK) )
        IF(PEK%XZ0(JI)/=XUNDEF)       PEK%XZ0(JI)       = PEK%XZ0(JI)     *( 1.+ S%XPERTZ0(IMASK) )
        IF(ISSK%XZ0EFFIP(JI)/=XUNDEF) ISSK%XZ0EFFIP(JI) = ISSK%XZ0EFFIP(JI)*( 1.+ S%XPERTZ0(IMASK) )
        IF(ISSK%XZ0EFFIM(JI)/=XUNDEF) ISSK%XZ0EFFIM(JI) = ISSK%XZ0EFFIM(JI)*( 1.+ S%XPERTZ0(IMASK) )
        IF(ISSK%XZ0EFFJP(JI)/=XUNDEF) ISSK%XZ0EFFJP(JI) = ISSK%XZ0EFFJP(JI)*( 1.+ S%XPERTZ0(IMASK) )
        IF(ISSK%XZ0EFFJM(JI)/=XUNDEF) ISSK%XZ0EFFJM(JI) = ISSK%XZ0EFFJM(JI)*( 1.+ S%XPERTZ0(IMASK) )
        !
     ENDDO
     !
  ENDDO
  !
ENDIF
!
! --------------------------------------------------------------------------------------
! Outputs for the atmospheric model or update the snow/flood fraction at time t+1
! --------------------------------------------------------------------------------------
! Grid box average fluxes/properties: Arguments and standard diagnostics at time t+1
!
CALL AVERAGE_FLUX(S%XPATCH, ZSFTH_TILE, ZSFTQ_TILE, ZSFTS_TILE,   &
                  ZSFCO2_TILE, ZSFU_TILE, ZSFV_TILE, PSFTH, PSFTQ,&
                  PSFTS, PSFCO2, PSFU, PSFV                       )  
!
! Get output megan flux if megan is activated


IF (CHI%SVI%NBEQ>0 .AND. CHI%LCH_BIO_FLUX) THEN
  IF (TRIM(CHI%CPARAMBVOC) == 'MEGAN') THEN
        ! Get output Isoprene flux
        IF (MGN%CMECHANISM == "RELACS") THEN
         DO II=1,SIZE(MGN%XBIOFLX,1)
            IF ((S%XPATCH(II,1) + S%XPATCH(II,2) + S%XPATCH(II,3)) .LT. 1.) THEN
               MGN%XBIOFLX(II) = PSFTS(II,MGN%NBIO)/(1. - S%XPATCH(II,1) - S%XPATCH(II,2) - S%XPATCH(II,3))
            ELSE
               MGN%XBIOFLX(:) = PSFTS(:,MGN%NBIO)
            ENDIF
         ENDDO
        END IF 
        IF ((MGN%CMECHANISM == "RELACS2").OR.(MGN%CMECHANISM == "CACM")) THEN
         DO II=1,SIZE(MGN%XBIOFLX,1)
            IF ((S%XPATCH(II,1) + S%XPATCH(II,2) + S%XPATCH(II,3)) .LT. 1.) THEN
               MGN%XBIOFLX(II) = PSFTS(II,MGN%NISOP)/(1. - S%XPATCH(II,1) - S%XPATCH(II,2) - S%XPATCH(II,3))
            ELSE
               MGN%XBIOFLX(:) = PSFTS(:,MGN%NISOP)
            ENDIF
         ENDDO
        END IF 
  ENDIF
ENDIF

!
!-------------------------------------------------------------------------------
!Physical properties see by the atmosphere in order to close the energy budget 
!between surfex and the atmosphere. All variables should be at t+1 but very 
!difficult to do. Maybe it will be done later. However, Ts is at time t+1
!-------------------------------------------------------------------------------
!   
CALL AVERAGE_PHY(S%XPATCH, ZTSURF_TILE, ZZ0_TILE, ZZ0H_TILE,          &
                 ZQSURF_TILE, PUREF, PZREF, PTSURF, PZ0, PZ0H, PQSURF )
!
!-------------------------------------------------------------------------------------
!Radiative properties at time t+1 (see by the atmosphere) in order to close
!the energy budget between surfex and the atmosphere
!-------------------------------------------------------------------------------------
!
DO JP = 1,IO%NPATCH
   CALL UPDATE_RAD_ISBA_n(IO, S, NK%AL(JP), NP%AL(JP), NPE%AL(JP), JP, PZENITH2, PSW_BANDS,  &
                          DTI%NPAR_VEG_IRR_USE, ZDIR_ALB_TILE(:,:,JP), ZSCA_ALB_TILE(:,:,JP),&
                          ZEMIS_TILE(:,JP), ZRNSHADE, ZRNSUNLIT, PDIR_SW, PSCA_SW)
ENDDO
!
!
CALL AVERAGE_RAD(S%XPATCH, ZDIR_ALB_TILE, ZSCA_ALB_TILE, ZEMIS_TILE, &
                 ZTRAD_TILE, PDIR_ALB, PSCA_ALB, S%XEMIS_NAT, S%XTSRAD_NAT  )  
!
PEMIS = S%XEMIS_NAT
PTRAD = S%XTSRAD_NAT
!
!-------------------------------------------------------------------------------------
!
! Any additional diagnostics (stored in MODD_DIAG_ISBA_n)
!
CALL AVERAGE_DIAG_ISBA_n(IO, ID%O, ID%D, ID%DC, ID%ND, ID%NDC, NP,&
                         PUREF, PZREF, PSFCO2                     )
!
! Water and energy fluxes diagnostics (stored in MODD_DIAG_EVAP_ISBA_n)
!
CALL AVERAGE_DIAG_EVAP_ISBA_n(IO, ID%O, ID%DE, ID%DEC, ID%NDE, ID%NDEC, NP,&
                              PTSTEP, PRAIN, PSNOW                         )
!
! Carbon fluxes diagnostics (stored in MODD_DIAG_EVAP_ISBA_n)
!
CALL AVERAGE_DIAG_CC_ISBA_n(IO, ID%O, ID%DE, ID%DEC, ID%NDE, ID%NDEC, NP)
!
! Miscellaneous diagnostics (stored in MODD_DIAG_MISC_ISBA_n)
!
IF (ID%DM%LSURF_MISC_BUDGET) THEN
   CALL AVERAGE_DIAG_MISC_ISBA_n(ID%DM, ID%NDM, IO, NK, NP, NPE)
ENDIF
!
!--------------------------------------------------------------------------------------
!
CALL COUPLING_SURF_TOPD(ID%DE, ID%DEC, ID%DC, ID%DM, IG, &
                        IO, S, K, NK, NP, NPE, UG, U, HPROGRAM, U%NDIM_FULL)
!
! --------------------------------------------------------------------------------------
! Snow/Flood fractions, albedo and emissivity update :
! --------------------------------------------------------------------------------------
!
! --------------------------------------------------------------------------------------
! Chemical fluxes :
! --------------------------------------------------------------------------------------
!
IF (CHI%SVI%NBEQ>0 .AND. CHI%LCH_BIO_FLUX .AND. TRIM(CHI%CPARAMBVOC)=='SOLMON' ) THEN
    CALL CH_BVOCEM_n(CHI%SVI, NGB, GB, IO, S, NP, NPE, ZSW_FORBIO, PRHOA, DTI%NPAR_VEG_IRR_USE, PSFTS)
ENDIF
!
!SOILNOX
IF (CHI%LCH_NO_FLUX) THEN
  CALL SOILEMISNO_n(GB, S, K, NP, NPE, PU, PV)
ENDIF
!
!==========================================================================================
!
IF (LHOOK) CALL DR_HOOK('COUPLING_ISBA_N',1,ZHOOK_HANDLE)
CONTAINS
!
!=======================================================================================
SUBROUTINE TREAT_PATCH(KK, PK, PEK, ISSK, AGK, GK, CHIK, DSTK, DST, SLT, DK, DCK, DEK, DECK, DMK, GBK, LIS_TREE)
!
USE MODD_ISBA_n, ONLY : ISBA_K_t, ISBA_P_t, ISBA_PE_t
USE MODD_SFX_GRID_n, ONLY : GRID_t
USE MODD_SSO_n, ONLY : SSO_t
USE MODD_AGRI_n, ONLY : AGRI_t
USE MODD_CH_ISBA_n, ONLY : CH_ISBA_t
USE MODD_DST_n, ONLY : DST_t
USE MODD_SLT_n, ONLY : SLT_t
USE MODD_DIAG_n, ONLY : DIAG_t
USE MODD_DIAG_EVAP_ISBA_n, ONLY : DIAG_EVAP_ISBA_t
USE MODD_DIAG_MISC_ISBA_n, ONLY : DIAG_MISC_ISBA_t
USE MODD_GR_BIOG_n, ONLY : GR_BIOG_t
!
IMPLICIT NONE
!
TYPE(ISBA_K_t),         INTENT(INOUT) :: KK
TYPE(ISBA_P_t),         INTENT(INOUT) :: PK
TYPE(ISBA_PE_t),        INTENT(INOUT) :: PEK
TYPE(SSO_t),            INTENT(INOUT) :: ISSK
TYPE(AGRI_t),           INTENT(INOUT) :: AGK
TYPE(GRID_t),           INTENT(INOUT) :: GK
TYPE(CH_ISBA_t),        INTENT(INOUT) :: CHIK
TYPE(DST_t),            INTENT(INOUT) :: DSTK, DST
TYPE(SLT_t),            INTENT(INOUT) :: SLT
TYPE(DIAG_t),           INTENT(INOUT) :: DK
TYPE(DIAG_t),           INTENT(INOUT) :: DCK
TYPE(DIAG_EVAP_ISBA_t), INTENT(INOUT) :: DEK
TYPE(DIAG_EVAP_ISBA_t), INTENT(INOUT) :: DECK
TYPE(DIAG_MISC_ISBA_t), INTENT(INOUT) :: DMK
TYPE(GR_BIOG_t),        INTENT(INOUT) :: GBK
!
LOGICAL, INTENT(IN) :: LIS_TREE
!
REAL, DIMENSION(PK%NSIZE_P) :: ZP_ZREF    ! height of T,q forcing                 (m)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_UREF    ! height of wind forcing                (m)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_U       ! zonal wind                            (m/s)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_V       ! meridian wind                         (m/s)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_WIND    ! wind                                  (m/s)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_DIR     ! wind direction                        (rad from N clockwise)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_QA      ! air specific humidity forcing         (kg/kg)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_TA      ! air temperature forcing               (K)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_CO2     ! CO2 concentration in the air          (kg/kg)
REAL, DIMENSION(PK%NSIZE_P,NIMPUR) :: ZP_IMPWET
REAL, DIMENSION(PK%NSIZE_P,NIMPUR) :: ZP_IMPDRY 
REAL, DIMENSION(PK%NSIZE_P,KSV) :: ZP_SV      ! scalar concentration in the air       (kg/kg)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_ZENITH  ! zenithal angle        radian from the vertical)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_PEW_A_COEF ! implicit coefficients
REAL, DIMENSION(PK%NSIZE_P) :: ZP_PEW_B_COEF ! needed if HCOUPLING='I'
REAL, DIMENSION(PK%NSIZE_P) :: ZP_PET_A_COEF
REAL, DIMENSION(PK%NSIZE_P) :: ZP_PET_B_COEF
REAL, DIMENSION(PK%NSIZE_P) :: ZP_PEQ_A_COEF
REAL, DIMENSION(PK%NSIZE_P) :: ZP_PEQ_B_COEF
REAL, DIMENSION(PK%NSIZE_P) :: ZP_RAIN    ! liquid precipitation                  (kg/m2/s)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_SNOW    ! snow precipitation                    (kg/m2/s)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_LW      ! longwave radiation (W/m2)
REAL, DIMENSION(PK%NSIZE_P,IO%NGROUND_LAYER) :: ZLE_HVEG   ! latent heat flux extracted by high veg (for TEB trees) (W/m2) 
REAL, DIMENSION(PK%NSIZE_P,ISWB) :: ZP_DIR_SW  ! direct  solar radiation (W/m2)
REAL, DIMENSION(PK%NSIZE_P,ISWB) :: ZP_SCA_SW  ! diffuse solar radiation (W/m2)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_PS      ! pressure at atmospheric model surface (Pa)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_PA      ! pressure at forcing level             (Pa)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_ZS      ! atmospheric model orography           (m)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_SFTQ    ! flux of water vapor <w'q'>            (kg.m-2.s-1)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_SFTH    ! flux of temperature <w'T'>            (W/m2)
REAL, DIMENSION(PK%NSIZE_P,KSV) :: ZP_SFTS    ! flux of scalar      <w'sv'>           (mkg/kg/s)
REAL, DIMENSION(PK%NSIZE_P) :: ZF_DSTI, ZF_DSTJ 
REAL, DIMENSION(PK%NSIZE_P) :: ZP_SFCO2   ! flux of CO2 positive toward the atmosphere (m/s*kg_CO2/kg_air)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_USTAR   ! friction velocity                     (m/s)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_SFU     ! zonal momentum flux                   (pa)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_SFV     ! meridian momentum flux                (pa)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_TRAD    ! radiative temperature                 (K)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_TSURF   ! surface effective temperature (K)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_Z0      ! roughness length for momentum (m)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_Z0H     ! roughness length for heat     (m)
REAL, DIMENSION(PK%NSIZE_P):: ZP_QSURF   ! specific humidity at surface  (kg/kg)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_TEMP, ZP_PAR

!
!*  other forcing variables (packed for each patch)
!
REAL, DIMENSION(PK%NSIZE_P) :: ZP_RHOA    ! lowest atmospheric level air density          (kg/m3)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_EXNA    ! Exner function at lowest atmospheric level    (-)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_EXNS    ! Exner function at surface                     (-)
REAL, DIMENSION(PK%NSIZE_P) :: ZP_ALFA    ! Wind direction   (-)
!
!*  working variables (packed for each patch)
!
REAL, DIMENSION(PK%NSIZE_P)      :: ZP_ALBNIR_TVEG         ! total vegetation albedo in ir
REAL, DIMENSION(PK%NSIZE_P)      :: ZP_ALBNIR_TSOIL        ! total soil albedo in ir
REAL, DIMENSION(PK%NSIZE_P)      :: ZP_ALBVIS_TVEG         ! total vegetation albedo in vis
REAL, DIMENSION(PK%NSIZE_P)      :: ZP_ALBVIS_TSOIL        ! total soil albedo in vis
REAL, DIMENSION(PK%NSIZE_P) :: ZP_EMIS                      ! emissivity
REAL, DIMENSION(PK%NSIZE_P) :: ZP_GLOBAL_SW                 ! global incoming SW rad.
REAL, DIMENSION(PK%NSIZE_P) :: ZP_SLOPE_COS                 ! typical slope in the grid cosine
REAL, DIMENSION(PK%NSIZE_P) :: ZP_SLOPE_DIR                 ! typical slope aspect in the grid 
!
REAL, DIMENSION(PK%NSIZE_P) :: ZP_Z0FLOOD  !Floodplain 
REAL, DIMENSION(PK%NSIZE_P) :: ZP_FFGNOS   !Floodplain fraction over the ground without snow
REAL, DIMENSION(PK%NSIZE_P) :: ZP_FFVNOS   !Floodplain fraction over vegetation without snow
!
REAL, DIMENSION(:,:),ALLOCATABLE :: ZP_PFT
REAL, DIMENSION(:,:),ALLOCATABLE :: ZP_EF
REAL, DIMENSION(:), ALLOCATABLE  :: ZP_T24, ZP_PFD24
INTEGER, DIMENSION(PK%NSIZE_P) :: IP_SLTYP
!
REAL, DIMENSION(PK%NSIZE_P,IO%NNBIOMASS) :: ZP_RESP_BIOMASS_INST         ! instantaneous biomass respiration (kgCO2/kgair m/s)
!
!*  Aggregated coeffs for evaporative flux calculations
!
REAL, DIMENSION(PK%NSIZE_P) :: ZP_AC_AGG      ! aggregated aerodynamic resistance
REAL, DIMENSION(PK%NSIZE_P) :: ZP_HU_AGG      ! aggregated relative humidity
!
!*  For multi-energy balance
!
REAL, DIMENSION(PK%NSIZE_P) :: ZPALPHAN                     ! snow/canopy transition coefficient
REAL, DIMENSION(PK%NSIZE_P) :: ZSNOWDEPTH                   ! total snow depth
REAL, DIMENSION(PK%NSIZE_P) :: ZZ0G_WITHOUT_SNOW            ! roughness length for momentum at snow-free canopy floor
REAL, DIMENSION(PK%NSIZE_P) :: ZZ0_MEBV                     ! roughness length for momentum over MEB vegetation part of patch
REAL, DIMENSION(PK%NSIZE_P) :: ZZ0H_MEBV                    ! roughness length for heat over MEB vegetation part of path
REAL, DIMENSION(PK%NSIZE_P) :: ZZ0EFF_MEBV                  ! roughness length for momentum over MEB vegetation part of patch
REAL, DIMENSION(PK%NSIZE_P) :: ZZ0_MEBN                     ! roughness length for momentum over MEB snow part of patch
REAL, DIMENSION(PK%NSIZE_P) :: ZZ0H_MEBN                    ! roughness length for heat over MEB snow part of path
REAL, DIMENSION(PK%NSIZE_P) :: ZZ0EFF_MEBN                  ! roughness length for momentum over MEB snow part of patch
! Temporary
REAL, DIMENSION(PK%NSIZE_P) :: ZP_MEB_SCA_SW                ! diffuse incoming SW rad.
!
!*  ISBA water and energy budget
!
REAL, DIMENSION(PK%NSIZE_P) :: ZP_WG_INI
REAL, DIMENSION(PK%NSIZE_P) :: ZP_WGI_INI
REAL, DIMENSION(PK%NSIZE_P) :: ZP_WR_INI
REAL, DIMENSION(PK%NSIZE_P) :: ZP_SWE_INI
!
REAL, DIMENSION(PK%NSIZE_P) :: ZP_RNSHADE
REAL, DIMENSION(PK%NSIZE_P) :: ZP_RNSUNLIT
!
! miscellaneous
!
REAL, DIMENSION(PK%NSIZE_P) :: ZP_DEEP_FLUX ! Flux at the bottom of the soil
REAL, DIMENSION(PK%NSIZE_P) :: ZP_TDEEP_A   ! coefficient for implicitation of Tdeep
REAL, DIMENSION(PK%NSIZE_P) :: ZIRRIG_GR    ! green roof ground irrigation rate 
!
REAL, DIMENSION(PK%NSIZE_P) :: ZP_AZIM  ! azimuthal angle      (radian from North, clockwise)
!
! For multi-energy balance
LOGICAL :: GMEB  ! True if multi-energy balance should be used for the specific patch
!
INTEGER :: JJ, JI, JK, JIMP
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
IF (LHOOK) CALL DR_HOOK('COUPLING_ISBA_n:TREAT_PATCH',0,ZHOOK_HANDLE)
!
IF (ASSOCIATED(MGN%XPFT)) THEN
  ALLOCATE(ZP_PFT(SIZE(MGN%XPFT,1),PK%NSIZE_P))
ELSE
  ALLOCATE(ZP_PFT(0,0))
ENDIF
IF (ASSOCIATED(MGN%XEF)) THEN
  ALLOCATE(ZP_EF(SIZE(MGN%XEF,1),PK%NSIZE_P))
ELSE
  ALLOCATE(ZP_EF(0,0))
ENDIF
IF (ASSOCIATED(MGN%XPPFD24)) THEN
  ALLOCATE(ZP_PFD24(PK%NSIZE_P))
ELSE
  ALLOCATE(ZP_PFD24(0))
ENDIF
IF (ASSOCIATED(MGN%XT24)) THEN
  ALLOCATE(ZP_T24(PK%NSIZE_P))
ELSE
  ALLOCATE(ZP_T24(0))
ENDIF
!--------------------------------------------------------------------------------------
!
! Pack isba forcing outputs
!
IF (IO%NPATCH==1) THEN
   ZP_ZENITH(:)     = PZENITH     (:)
   ZP_AZIM(:)       = PAZIM       (:)
   ZP_ZREF(:)       = PZREF       (:)
   ZP_UREF(:)       = PUREF       (:)
   ZP_WIND(:)       = ZWIND       (:)
   ZP_U(:)          = PU          (:)
   ZP_V(:)          = PV          (:)
   ZP_DIR(:)        = ZDIR        (:)
   ZP_QA(:)         = ZQA         (:)
   ZP_TA(:)         = PTA         (:)
   ZP_CO2(:)        = ZCO2        (:)
#ifdef SFX_OL
   IF (LFORCIMP) THEN
    ZP_IMPWET(:,:)   = PIMPWET(:,:)
    ZP_IMPDRY(:,:)   = PIMPDRY(:,:)
   ENDIF
#endif
   ZP_SV(:,:)       = PSV         (:,:)
   ZP_PEW_A_COEF(:) = PPEW_A_COEF (:)
   ZP_PEW_B_COEF(:) = PPEW_B_COEF (:)
   ZP_PET_A_COEF(:) = PPET_A_COEF (:)
   ZP_PET_B_COEF(:) = PPET_B_COEF (:)
   ZP_PEQ_A_COEF(:) = ZPEQ_A_COEF (:)
   ZP_PEQ_B_COEF(:) = ZPEQ_B_COEF (:)
   ZP_RAIN(:)       = PRAIN       (:)
   ZP_SNOW(:)       = PSNOW       (:)
   ZP_LW(:)         = PLW         (:)
   ZP_DIR_SW(:,:)   = PDIR_SW     (:,:)
   ZP_SCA_SW(:,:)   = PSCA_SW     (:,:)
   ZP_PS(:)         = PPS         (:)
   ZP_PA(:)         = PPA         (:)
   ZP_ZS(:)         = PZS         (:)
!
   ZP_RHOA(:)       = PRHOA       (:)
   ZP_EXNA(:)       = ZEXNA       (:)
   ZP_EXNS(:)       = ZEXNS       (:)
   ZP_ALFA(:)       = ZALFA       (:)

   IF ((TRIM(CHI%CPARAMBVOC) == 'MEGAN') .AND. CHI%LCH_BIO_FLUX) THEN
      ZP_PFT(:,:)  = MGN%XPFT  (:,:)
      ZP_EF(:,:)   = MGN%XEF   (:,:)
      IP_SLTYP(:)  = MGN%NSLTYP  (:)
      ZP_PFD24(:)  = MGN%XPPFD24 (:)
      ZP_T24(:)    = MGN%XT24 (:)
   END IF   
   
   ZP_RNSHADE(:)    = ZRNSHADE    (:)
   ZP_RNSUNLIT(:)   = ZRNSUNLIT   (:)

ELSE
!cdir nodep
!cdir unroll=8
  DO JJ=1,PK%NSIZE_P
   JI = PK%NR_P(JJ)
   ZP_ZENITH(JJ)     = PZENITH     (JI)
   ZP_AZIM(JJ)       = PAZIM       (JI)   
   ZP_ZREF(JJ)       = PZREF       (JI)
   ZP_UREF(JJ)       = PUREF       (JI)
   ZP_WIND(JJ)       = ZWIND       (JI)
   ZP_U(JJ)          = PU          (JI)
   ZP_V(JJ)          = PV          (JI)
   ZP_DIR(JJ)        = ZDIR        (JI)
   ZP_QA(JJ)         = ZQA         (JI)
   ZP_TA(JJ)         = PTA         (JI)
   ZP_CO2(JJ)        = ZCO2        (JI)
   ZP_PEW_A_COEF(JJ) = PPEW_A_COEF (JI)
   ZP_PEW_B_COEF(JJ) = PPEW_B_COEF (JI)
   ZP_PET_A_COEF(JJ) = PPET_A_COEF (JI)
   ZP_PET_B_COEF(JJ) = PPET_B_COEF (JI)
   ZP_PEQ_A_COEF(JJ) = ZPEQ_A_COEF (JI)
   ZP_PEQ_B_COEF(JJ) = ZPEQ_B_COEF (JI)
   ZP_RAIN(JJ)       = PRAIN       (JI)
   ZP_SNOW(JJ)       = PSNOW       (JI)
   ZP_LW(JJ)         = PLW         (JI)
   ZP_PS(JJ)         = PPS         (JI)
   ZP_PA(JJ)         = PPA         (JI)
   ZP_ZS(JJ)         = PZS         (JI)
!
   ZP_RHOA(JJ)       = PRHOA       (JI)
   ZP_EXNA(JJ)       = ZEXNA       (JI)
   ZP_EXNS(JJ)       = ZEXNS       (JI)
   ZP_ALFA(JJ)       = ZALFA       (JI)
  ENDDO
!
  DO JK=1,KSV
!cdir nodep
!cdir unroll=8
    DO JJ=1,PK%NSIZE_P
      JI=PK%NR_P(JJ)
      ZP_SV(JJ,JK) = PSV(JI,JK)
    ENDDO
  ENDDO
!
  DO JK=1,SIZE(PDIR_SW,2)
!cdir nodep
!cdir unroll=8
    DO JJ=1,PK%NSIZE_P
      JI=PK%NR_P(JJ)
      ZP_DIR_SW(JJ,JK) = PDIR_SW (JI,JK)
      ZP_SCA_SW(JJ,JK) = PSCA_SW (JI,JK)
    ENDDO
  ENDDO
!
#ifdef SFX_OL
  IF (LFORCIMP) THEN
    DO JIMP=1,NIMPUR
      DO JJ=1,PK%NSIZE_P
        JI = PK%NR_P(JJ)
        ZP_IMPWET(JJ,JIMP)   = PIMPWET(JI,JIMP)
      ENDDO
    ENDDO
  ENDIF
#endif

  IF ((TRIM(CHI%CPARAMBVOC) == 'MEGAN') .AND. CHI%LCH_BIO_FLUX) THEN  
    DO JJ=1,PK%NSIZE_P
      JI=PK%NR_P(JJ)
      ZP_PFT(:,JJ) = MGN%XPFT  (:,JI)
      ZP_EF(:,JJ)  = MGN%XEF   (:,JI)
      IP_SLTYP(JJ) = MGN%NSLTYP  (JI)
      ZP_PFD24(JJ) = MGN%XPPFD24 (JI)
      ZP_T24(JJ)   = MGN%XT24 (JI)
    ENDDO
  END IF
  DO JJ=1,PK%NSIZE_P
    JI=PK%NR_P(JJ)
    ZP_RNSHADE(JJ)  = ZRNSHADE (JI)
    ZP_RNSUNLIT(JJ) = ZRNSUNLIT(JI)
  ENDDO 
ENDIF
!
!--------------------------------------------------------------------------------------
!
! For multi-energy balance
GMEB = IO%LMEB_PATCH(JP)
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Cosine of the slope typically encoutered in the grid mesh (including subgrid orography)
!  and orientation of this slope
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
ZP_SLOPE_COS(:) = 1./SQRT(1.+ISSK%XSSO_SLOPE(:)**2)
ZP_SLOPE_DIR(:) = +ISSK%XSSO_DIR(:)
IF(LNOSOF .AND. (.NOT. LSLOPE)) THEN
    ZP_SLOPE_COS(:) = 1.0
    ZP_SLOPE_DIR(:) = -1.0
ENDIF
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Snow fractions
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! now caculated at the initialization and at the end of the time step 
! (see update_frac_alb_emis_isban.f90) in order to close the energy budget
! between surfex and the atmosphere. This fact do not change the offline runs.
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Blowing snow scheme
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
IF(CHI%SVI%NSNWEQ>0.) THEN
   ZP_BLOWSNW_CONC(:,:) =  ZP_SV(:,CHI%SVI%NSV_SNWBEG:CHI%SVI%NSV_SNWEND)
   ZP_BLOWSNW_FLUX(:,:) =  ZP_SV(:,CHI%SVI%N2D_SNWBEG:CHI%SVI%N2D_SNWEND)
!  ZP_BLOWSNW_FLUX           IN : fluxes sent from Canopy:
!                                         [1] number sedim. flux (#/m2/s)
!                                         [2] mass sedim flux (kg{snow}/m2/s)
!                                         [3] contrib. saltation (kg{snow}/m2/s)
!                            OUT : fluxes towards Canopy:
!                                         [1] number turbulent flux (#/m2/s)
!                                         [2] mass turbulent flux (kg{snow}/m2/s)
!                                         [3] updated streamwise saltation flux (kg{snow}/m2/s)
ELSE
   ZP_BLOWSNW_CONC(:,:) =  XUNDEF
   ZP_BLOWSNW_FLUX(:,:) =  XUNDEF
END IF
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! No implicitation of Tdeep
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
ZP_TDEEP_A = 0.
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Flood properties 
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
IF(IO%LFLOOD)THEN
  CALL ISBA_FLOOD_PROPERTIES(PEK%XLAI, KK%XFFLOOD, KK%XFFROZEN, ZP_Z0FLOOD, ZP_FFGNOS, ZP_FFVNOS)  
ELSE
  ZP_Z0FLOOD = XUNDEF
  ZP_FFGNOS  = 0.0
  ZP_FFVNOS  = 0.0
ENDIF
!
! For multi-energy balance
   IF(GMEB)THEN
     ZSNOWDEPTH(:) = SUM(PEK%TSNOW%WSNOW(:,:)/PEK%TSNOW%RHO(:,:),2)
     ZPALPHAN  (:)  =MEBPALPHAN(ZSNOWDEPTH,PEK%XH_VEG(:))
   ENDIF
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Surface Roughness lengths (m):
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
!* effective roughness
!
CALL Z0EFF(PEK%TSNOW%SCHEME, GMEB, ZP_ALFA, ZP_ZREF, ZP_UREF,             &
           PEK%XZ0, ISSK%XZ0REL, PEK%XPSN, ZPALPHAN, PEK%XZ0LITTER,       &
           PEK%TSNOW%WSNOW(:,1), ISSK, KK%XFF, ZP_Z0FLOOD, PK%XZ0_O_Z0H,  &
           DK%XZ0, DK%XZ0H, DK%XZ0EFF, ZZ0G_WITHOUT_SNOW,                 &
           ZZ0_MEBV, ZZ0H_MEBV, ZZ0EFF_MEBV, ZZ0_MEBN, ZZ0H_MEBN, ZZ0EFF_MEBN )
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Shortwave computations for outputs (albedo for radiative scheme)
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! now caculated at the initialization and at the end of the time step 
! (see update_frac_alb_emis_isban.f90) in order to close the energy budget
! between surfex and the atmosphere. This fact do not change the offline runs.
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Shortwave computations for ISBA inputs (global snow-free albedo)
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
! ISBA needs global incoming solar radiation: it currently does
! not distinguish between the scattered and direct components,
! or between different wavelengths.
!
!
!* Snow-free surface albedo for each wavelength
!
CALL ISBA_ALBEDO(PEK, IO%LTR_ML, GMEB, ZP_DIR_SW, ZP_SCA_SW,                  &
                 PSW_BANDS, ISWB, KK%XALBF, KK%XFFV, KK%XFFG, ZP_GLOBAL_SW,   &
                 ZP_MEB_SCA_SW, ZP_ALBNIR_TVEG, ZP_ALBVIS_TVEG,               &
                 ZP_ALBNIR_TSOIL, ZP_ALBVIS_TSOIL, PSNFREE_SWU=DEK%XSNFREE_SWU)  
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Nudge the snow
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
IF (IO%LNUDG_SWE) THEN
   CALL NUDGING_ISBA_SNOW(IO, KK, PK, PEK, DMK, S%TTIME%TDATE%YEAR, KMONTH, KDAY, PTSTEP, ZP_SNOW)
ENDIF
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Intialize computation of ISBA water and energy budget
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
CALL ISBA_BUDGET_INIT(ID%DE%LWATER_BUDGET, IO%CISBA, PEK, PK%XDG, PK%XDZG, &
                     ZP_WG_INI, ZP_WGI_INI, ZP_WR_INI, ZP_SWE_INI  )
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Over Natural Land Surfaces:
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
ZIRRIG_GR(:)= 0.
ZLE_HVEG(:,:) = 0.
!
CALL ISBA(IO, KK, PK, PEK, GK, AGK, DK, DEK, DMK, S%TTIME, S%XPOI, S%XABC,     &
           GBK%XIACAN, GMEB, PTSTEP, CIMPLICIT_WIND, ZP_ZREF, ZP_UREF,         &
           ZP_SLOPE_COS, IO%XCVHEATF, ZP_SLOPE_DIR,ZP_IMPWET,ZP_IMPDRY,        &
           ZP_TA, ZP_QA, ZP_EXNA, ZP_RHOA, ZP_PS, ZP_EXNS, ZP_RAIN,            &
           ZP_SNOW, ZP_ZENITH,ZP_AZIM, ZP_MEB_SCA_SW, ZP_GLOBAL_SW, ZP_LW,     &
           ZLE_HVEG, ZP_WIND, ZP_DIR, ZP_PEW_A_COEF, ZP_PEW_B_COEF, ZP_PET_A_COEF,       &
           ZP_PEQ_A_COEF, ZP_PET_B_COEF, ZP_PEQ_B_COEF, AT, ZP_ALBNIR_TVEG,    &
           ZP_ALBVIS_TVEG, ZP_ALBNIR_TSOIL, ZP_ALBVIS_TSOIL, ZPALPHAN,         &
           ZZ0G_WITHOUT_SNOW, ZZ0_MEBV, ZZ0H_MEBV, ZZ0EFF_MEBV,                &
           ZZ0_MEBN, ZZ0H_MEBN, ZZ0EFF_MEBN, ZP_TDEEP_A, ZP_CO2, ZP_FFGNOS,    &
           ZP_FFVNOS, ZP_EMIS, ZP_USTAR, ZP_AC_AGG, ZP_HU_AGG,                 &
           ZP_RESP_BIOMASS_INST, ZP_DEEP_FLUX, ZIRRIG_GR, DTI%NPAR_VEG_IRR_USE,&
           NTAB_SYT, ZP_DIR_SW, ZP_SCA_SW,                                     &
           ZP_RNSHADE, ZP_RNSUNLIT, ZP_BLOWSNW_FLUX, ZP_BLOWSNW_CONC)
!
ZP_TRAD = DK%XTSRAD
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Glacier : ice runoff flux (especally for Earth System Model)
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
IF (IO%LGLACIER) THEN
  !
  CALL HYDRO_GLACIER(PTSTEP, ZP_SNOW, PEK, DEK%XICEFLUX)
  !
ENDIF
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Calculation of ISBA water and energy budget (and time tendencies of each reservoir)
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
CALL ISBA_BUDGET(IO, PK, PEK, DK, DEK, ID%DE, PTSTEP, ZP_WG_INI,     &
                 ZP_WGI_INI, ZP_WR_INI, ZP_SWE_INI, ZP_RAIN, ZP_SNOW )
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Evolution of soil albedo, when depending on surface soil wetness:
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
IF (IO%CALBEDO=='EVOL' .AND. IO%LECOCLIMAP) THEN
  !
  CALL SOIL_ALBEDO(IO%CALBEDO, KK%XWSAT(:,1),PEK%XWG(:,1), KK, PEK, "ALL")  
  !
  CALL ALBEDO(IO%CALBEDO,PEK)  
  !
END IF
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Vegetation evolution for interactive LAI
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
IF (IO%CPHOTO=='NIT' .OR. IO%CPHOTO=='NCB') THEN
  !
  CALL VEGETATION_EVOL(IO, DTI, PK, PEK, DEK, LAGRIP, PTSTEP, KMONTH, KDAY, PTIME, &
                       GK%XLAT, ZP_CO2, ISSK, ZP_RESP_BIOMASS_INST,(.NOT.LIS_TREE) ) 
  !
ELSEIF ( LIRRIGMODE .AND. LMULTI_SEASON ) THEN
  !
  ! Update irrigation only if no interavtive vegetation  
  CALL VEGETATION_EVOL(IO, DTI, PK, PEK, DEK, LAGRIP, PTSTEP, KMONTH, KDAY, PTIME, GK%XLAT, &
                      ZP_CO2, ISSK, ZP_RESP_BIOMASS_INST,(.NOT.LIS_TREE),OIRRIG_ONLY=.TRUE. )
  !
END IF
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Diagnostic of respiration carbon fluxes and soil carbon evolution
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
ZP_SFCO2      (:)  = 0.
DEK%XRESP_ECO (:)  = 0.
DEK%XRESP_AUTO(:)  = 0.
DEK%XTURNVTOT (:)  = 0.
DEK%XFDOC     (:)  = 0.
DEK%XFDOCLIT  (:)  = 0.
DEK%XFLTOSCARB(:)  = 0.
DEK%XRESPSCARB(:)  = 0.
DEK%XRESPLIT  (:)  = 0.
!
DMK%XTSOILPOOL(:,:)= 0.
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Vegetation Disturbance due to wild fire
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
IF (IO%LFIRE) THEN
   CALL FIRE_DISTURBANCE(IO, KK, PK, PEK, DEK, DMK, HPROGRAM, PTSTEP, PTIME)
ENDIF
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Soil carbon evolution
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
IF ( IO%CPHOTO/='NON' .AND. IO%CRESPSL/='NON' .AND. ANY(PEK%XLAI(:)/=XUNDEF) ) THEN
   !
   IF ( IO%CRESPSL=='DIF')THEN
      !
      !Multi-layer soil carbon and gas schemes
      !
      CALL CARBON_EVOL_DIF(IO, KK, PK, PEK, DK, DEK, DMK, HPROGRAM, PTSTEP,    & 
                           ZP_RHOA, ZP_CO2, ZP_PS, ZP_TA, ZP_RESP_BIOMASS_INST )           
      !
   ELSE
      !
      !Simple 1-layer soil carbon scheme
      !          
      CALL CARBON_EVOL(IO, KK, PK, PEK, DEK, DMK, PTSTEP, ZP_RESP_BIOMASS_INST)
      !
   ENDIF
   !
ENDIF
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Update of net carbon flux toward the atmosphere
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Note: Positive toward the atmosphere (kgCO2/kgAir m/s)
!
IF (IO%CPHOTO/='NON'.AND.ANY(PEK%XLAI(:)/=XUNDEF))THEN
   !
   ! Calculation of vegetation CO2 flux :
   ZP_SFCO2(:) = (DEK%XRESP_ECO(:)-DEK%XGPP(:))/ZP_RHOA(:) ! kgC/m2/s -> kgC02/kgAir m/s
   !
   ! Accouting for missing flux (required to satisfied keeling hypothesis at preindustrial state= net land-ocean co2 flux ~ 0.0)
   IF (IO%XMISSFCO2/=XUNDEF) THEN
      ZP_SFCO2(:) = ZP_SFCO2(:) + (IO%XMISSFCO2*XMCO2/XMC)/ZP_RHOA(:) ! kgC/m2/s -> kgC02/kgAir m/s
   ENDIF
   !
   ! Accounting for natural fire co2 emissions (accounting for black carbon emissions) :
   IF (IO%LFIRE) THEN
      ZP_SFCO2(:) = ZP_SFCO2(:) + ((DEK%XFIRECO2(:)+DEK%XFIREBCS(:))/ZP_RHOA(:))*(XMCO2/XMC) ! kgC/m2/s -> kgC02/kgAir m/s 
   ENDIF
   !
ENDIF
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Reset effecitve roughness lentgh to its nominal value when snow has just disappeared
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
CALL SUBSCALE_Z0EFF(ISSK,PEK%XZ0(:),.FALSE.,OMASK=(PEK%TSNOW%WSNOW(:,1)==0. .AND. PEK%XPSN(:)>0.)  )   
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Turbulent fluxes
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
ZP_SFTH(:) = DK%XH(:)
ZP_SFTQ(:) = DK%XEVAP(:)

ZP_SFU (:) = 0.
ZP_SFV (:) = 0.
WHERE (ZP_WIND>0.)
  ZP_SFU (:) = - ZP_U(:)/ZP_WIND(:) * ZP_USTAR(:)**2 * ZP_RHOA(:)
  ZP_SFV (:) = - ZP_V(:)/ZP_WIND(:) * ZP_USTAR(:)**2 * ZP_RHOA(:)
END WHERE
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Scalar fluxes
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
ZP_SFTS(:,:) = 0.
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Blowing snow scheme
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
IF(CHI%SVI%NSNWEQ>0) THEN
! Store emitted turbulent flux  1: number (#/m2/s); 2: mass (kg/m2/s)
  ZP_SFTS(:,CHI%SVI%NSV_SNWBEG:CHI%SVI%NSV_SNWEND) = ZP_BLOWSNW_FLUX(:,1:CHI%SVI%NSNWEQ)
! Store streamwise saltation flux  (kg/m/s)
  ZP_SFTS(:,CHI%SVI%N2D_SNWEND) = ZP_BLOWSNW_FLUX(:,CHI%SVI%NSNWEQ+1)
END IF
!
! --------------------------------------------------------------------------------------
! Chemical dry deposition :
! --------------------------------------------------------------------------------------
!
IF (CHI%SVI%NBEQ>0) THEN
!
  ZP_SFTS(:,CHI%SVI%NSV_CHSBEG:CHI%SVI%NSV_CHSEND) = 0.
  ZP_SFTS(:,CHI%SVI%NSV_AERBEG:CHI%SVI%NSV_AEREND) = 0.        
!
  IF( CHI%CCH_DRY_DEP == "WES89") THEN
!
    IBEG = CHI%SVI%NSV_CHSBEG
    IEND = CHI%SVI%NSV_CHSEND 
    ISIZE = IEND - IBEG + 1 
!
    IF (ANY(PEK%XLAI(:)/=XUNDEF) ) THEN
      CALL CH_DEP_ISBA(KK, PK, PEK, DK, DMK, CHIK, &
                     ZP_USTAR, ZP_TA, ZP_PA, ZP_TRAD(:), ISIZE, DTI%NPAR_VEG_IRR_USE )  
 
      ZP_SFTS(:,IBEG:IEND) = - ZP_SV(:,IBEG:IEND) * CHIK%XDEP(:,1:CHI%SVI%NBEQ)  

      IF (CHI%SVI%NAEREQ > 0 ) THEN
       
        IBEG = CHI%SVI%NSV_AERBEG
        IEND = CHI%SVI%NSV_AEREND
        CALL CH_AER_DEP(ZP_SV(:,IBEG:IEND), ZP_SFTS(:,IBEG:IEND), ZP_USTAR, PEK%XRESA, ZP_TA, ZP_RHOA)     
      END IF
    ENDIF    
!
  ELSE
!
    IBEG = CHI%SVI%NSV_AERBEG
    IEND = CHI%SVI%NSV_AEREND
    ZP_SFTS(:,IBEG:IEND) = 0.
    ZP_SFTS(:,IBEG:IEND) = 0.
!
  ENDIF
ENDIF
!
! --------------------------------------------------------------------------------------
! Chemical natural flux (BVOC, NOx) from MEGAN:
! --------------------------------------------------------------------------------------
IF (CHI%SVI%NBEQ>0 .AND. CHI%LCH_BIO_FLUX) THEN
 IF ((TRIM(CHI%CPARAMBVOC) == 'MEGAN').AND.(ANY(PEK%XLAI(:)/=XUNDEF))) THEN

!UPG*PT
 WHERE (GBK%XIACAN > 2000.) ! non physical values
  GBK%XIACAN = 0.
 END WHERE
!UPG*PT
  IBEG = CHI%SVI%NSV_CHSBEG
  IEND = CHI%SVI%NSV_CHSEND 

 CALL COUPLING_MEGAN_n(MGN, CHI, GK, PEK, PTSTEP, &
                       KYEAR, KMONTH, KDAY, PTIME, S%TTIME%TIME, IO%LTR_ML, &
                       IP_SLTYP, ZP_PFT, ZP_EF, ZP_PFD24, ZP_T24,           &
                       ZP_TA, GBK%XIACAN, ZP_TRAD, ZP_RNSUNLIT, ZP_RNSHADE, &
                       ZP_WIND, ZP_PA, ZP_QA, ZP_SFTS(:,IBEG:IEND))

 END IF
ENDIF
! --------------------------------------------------------------------------------------
! Dust deposition and emission:
! --------------------------------------------------------------------------------------
!
IF(CHI%SVI%NDSTEQ>0)THEN
!
  IBEG = CHI%SVI%NSV_DSTBEG
  IEND = CHI%SVI%NSV_DSTEND
  IDST = IEND - IBEG + 1
!
!IRM 16/07/2019
  CALL COUPLING_DST_n(DSTK, DST, KK, PK, PEK, DK, &
            HPROGRAM,                    &!I [char] Name of program
            PK%NSIZE_P,      &!I [nbr] number of points in patch
            IDST,                        &!I [nbr] number of dust emissions variables
            ZP_PS,                       &!I [Pa] surface pressure
            ZP_QA,                       &!I [kg/kg] specific humidity
            ZP_RHOA,                     &!I [kg/m3] atmospheric density
            ZP_PA,                       &!I [K] Atmospheric pressure
            ZP_TA,                       &!I [K] Atmospheric temperature
            ZP_U,                        &!I [m/s] zonal wind at atmospheric height 
            ZP_UREF,                     &!I [m] reference height of wind
            ZP_V,                        &!I [m/s] meridional wind at atmospheric height
            ZP_ZREF,                     &!I [m] reference height of wind
            ZP_SFTS(:,IBEG:IEND)  &!O [kg/m2/sec] flux of dust            
            ) 
!IRM 16/07/2019 
!
   IF (CHI%SVI%NSV_DSTEND > 0)  THEN ! case of dust/ anthropogenic aerosols coupling

     DO JMODE=1,NDSTMDE
       !
       !Make index which is 0 for first mode, 3 for second, 6 for third etc
       IF (LVARSIG_DST) THEN
         JSV_IDX = (JMODE-1)*3
       ELSE IF (LRGFIX_DST) THEN
         JSV_IDX = JMODE-2
       ELSE
         JSV_IDX = (JMODE-1)*2
       END IF
       !
       IF (CRGUNIT=="MASS") THEN
        ZEMISRADIUSI = XEMISRADIUSI * EXP(-3.*(LOG(XEMISSIGI))**2)
        ZEMISRADIUSJ = XEMISRADIUSJ * EXP(-3.*(LOG(XEMISSIGJ))**2)
       ELSE
        ZEMISRADIUSI = XEMISRADIUSI
        ZEMISRADIUSJ = XEMISRADIUSJ
       END IF


       ZF_DSTI(:) = 0.
       ZF_DSTJ(:) = 0.
       DO JSV=1, size(HSV)  ! dust flux for orilam chemistry model
         IF ((TRIM(HSV(JSV)) == "@DSTI").AND.(JMODE==3)) THEN 
           ! dust mass flux in kg/m2/s
           ZF_DSTI(:) = ZP_SFTS(:,IBEG-1+JSV_IDX+2)
           ! add dust flux and conversion kg/m2/s into molec.m2/s
           ZP_SFTS(:,JSV) = ZP_SFTS(:,JSV) + ZF_DSTI(:)*XAVOGADRO/XMOLARWEIGHT_DST
           ! dust flux in moment 0
           ZF_DSTI(:) = ZF_DSTI(:) * 1E9 ! µg/m2/s
           ZF_DSTI(:) = ZF_DSTI(:) / ((4./3.)*3.14292654*XDENSITY_DST*1.e-9) !moment 3
           ZF_DSTI(:) = ZF_DSTI(:) / ((ZEMISRADIUSI**3)*EXP(4.5 * (LOG(XEMISSIGI))**2)) ! moment 0 molecules.m-2.s-1
         END IF
         IF (TRIM(HSV(JSV)) == "@M0I") ZP_SFTS(:,JSV) = ZP_SFTS(:,JSV) + ZF_DSTI(:)

         IF ( (TRIM(HSV(JSV)) == "@DSTJ").AND.(JMODE==2)) THEN 
           ! dust mass flux in kg/m2/s
           ZF_DSTJ(:) = ZP_SFTS(:,IBEG-1+JSV_IDX+2)
           ! add dust flux and conversion kg/m2/sec into molec.m2/s
           ZP_SFTS(:,JSV) = ZP_SFTS(:,JSV) + ZF_DSTJ(:)*XAVOGADRO/XMOLARWEIGHT_DST
           ! dust flux in moment 0
           ZF_DSTJ(:) = ZF_DSTJ(:) * 1E9 ! µg/m2/s
           ZF_DSTJ(:) = ZF_DSTJ(:) / ((4./3.)*3.14292654*XDENSITY_DST*1.e-9) !moment 3
           ZF_DSTJ(:) = ZF_DSTJ(:) / ((ZEMISRADIUSJ**3)*EXP(4.5 * (LOG(XEMISSIGJ))**2)) ! moment 0 molecules.m-2.s-1
         END IF
         IF (TRIM(HSV(JSV)) == "@M0J") ZP_SFTS(:,JSV) = ZP_SFTS(:,JSV) + ZF_DSTJ(:)
       END DO
       !
     END DO
    END IF
!    
!Modify fluxes due to dry deposition, we introduce a negative flux where dust is lost
  CALL DSLT_DEP(ZP_SV(:,IBEG:IEND), ZP_SFTS(:,IBEG:IEND), ZP_USTAR, PEK%XRESA,        &
                ZP_TA, ZP_RHOA, DST%XEMISSIG_DST, DST%XEMISRADIUS_DST, JPMODE_DST,  &
!                ZP_TA, ZP_RHOA, DSTK%XEMISSIG_DST, DSTK%XEMISRADIUS_DST, JPMODE_DST,  &
                XDENSITY_DST, XMOLARWEIGHT_DST, ZCONVERTFACM0_DST, ZCONVERTFACM6_DST, &
                ZCONVERTFACM3_DST, LVARSIG_DST, LRGFIX_DST, CVERMOD  )
!
!Transfer these fluxes to fluxes understandable by all moments
  CALL MASSFLUX2MOMENTFLUX(           &
    ZP_SFTS(:,IBEG:IEND),             & !I/O ![kg/m2/sec] In: flux of only mass, out: flux of moments
    ZP_RHOA,                          & !I [kg/m3] air density
    DST%XEMISRADIUS_DST,              & !I [um] emitted radius for the modes (max 3)
    DST%XEMISSIG_DST,                 & !I [-] emitted sigma for the different modes (max 3)
    NDSTMDE,                          &
    ZCONVERTFACM0_DST,                &
    ZCONVERTFACM6_DST,                &
    ZCONVERTFACM3_DST,                &
    LVARSIG_DST, LRGFIX_DST           )   
!
ENDIF !Check on CDSTYN
!
! --------------------------------------------------------------------------------------
! Sea Salt deposition
! --------------------------------------------------------------------------------------
!
IF (CHI%SVI%NSLTEQ>0) THEN
  !
  IBEG = CHI%SVI%NSV_SLTBEG
  IEND = CHI%SVI%NSV_SLTEND
  !
  CALL DSLT_DEP(ZP_SV(:,IBEG:IEND), ZP_SFTS(:,IBEG:IEND), ZP_USTAR, PEK%XRESA,         &
                ZP_TA, ZP_RHOA, SLT%XEMISSIG_SLT, SLT%XEMISRADIUS_SLT, JPMODE_SLT,     &
                XDENSITY_SLT, XMOLARWEIGHT_SLT, ZCONVERTFACM0_SLT, ZCONVERTFACM6_SLT,  &
                ZCONVERTFACM3_SLT, LVARSIG_SLT, LRGFIX_SLT, CVERMOD  )  

  CALL MASSFLUX2MOMENTFLUX(           &
    ZP_SFTS(:,IBEG:IEND),             & !I/O ![kg/m2/sec] In: flux of only mass, out: flux of moments
    ZP_RHOA,                          & !I [kg/m3] air density
    SLT%XEMISRADIUS_SLT,              & !I [um] emitted radius for the modes (max 3)
    SLT%XEMISSIG_SLT,                 & !I [-] emitted sigma for the different modes (max 3)
    NSLTMDE,                          &
    ZCONVERTFACM0_SLT,                &
    ZCONVERTFACM6_SLT,                &
    ZCONVERTFACM3_SLT,                &
    LVARSIG_SLT, LRGFIX_SLT         ) 
ENDIF !Check on CSLTYN
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Inline diagnostics
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
CALL DIAG_INLINE_ISBA_n(ID%O, KK, DK, IO%LCANOPY, ZP_TA, ZP_QA, ZP_PA,         &
                        ZP_PS, ZP_RHOA, ZP_U, ZP_V, ZP_ZREF, ZP_UREF, ZP_SFTH, &
                        ZP_SFTQ, ZP_SFU, ZP_SFV, ZP_DIR_SW, ZP_SCA_SW, ZP_LW )  
!
!
!-------------------------------------------------------------------------------
!Physical properties see by the atmosphere in order to close the energy budget 
!between surfex and the atmosphere. All variables should be at t+1 but very 
!difficult to do. Maybe it will be done later. However, Ts can be at time t+1
!-------------------------------------------------------------------------------
!
ZP_TSURF (:) = DK%XTS (:)
ZP_Z0    (:) = DK%XZ0 (:)
ZP_Z0H   (:) = DK%XZ0H(:)
ZP_QSURF (:) = DK%XQS (:)
!
!-------------------------------------------------------------------------------
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Isba offline diagnostics for each patch
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
IF(ID%O%LSURF_BUDGETC)THEN
!
!  Water and energy fluxes
   CALL DIAG_EVAP_CUMUL_ISBA_n(ID%DE, DECK, DCK, DEK, DK, PEK, &
                               IO, PTSTEP, PK%NSIZE_P, JP      )
!
!  Carbon fluxes
   CALL DIAG_CC_CUMUL_ISBA_n(DECK, DCK, DEK, DK, IO, PTSTEP, PK%NSIZE_P)
!
ENDIF
!
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! Isba offline diagnostics for miscellaneous terms over each patch
! - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
!
IF(ID%DM%LSURF_MISC_BUDGET)THEN
  CALL DIAG_MISC_ISBA_n(DMK, KK, PK, PEK, AGK, IO, ID%DM%LVOLUMETRIC_SNOWLIQ, PTSTEP, PTIME)
ENDIF
!
IF (PEK%TSNOW%SCHEME=='CRO') THEN
  CALL REPROJ_DIAG_ISBA_n(DK, DEK, DMK, PEK, ID%O%LSURF_BUDGET, ID%DE%LSURF_EVAP_BUDGET, &
                          ID%DE%LWATER_BUDGET, ID%DM%LSURF_MISC_BUDGET, ID%DM%LPROSNOW,  &
                          IO%LSNOWSYTRON, IO%LMEB_PATCH(JP), ZP_SLOPE_COS                )
ENDIF
!
IF (PEK%TSNOW%SCHEME=='3-L'.OR.PEK%TSNOW%SCHEME=='CRO') THEN
  PEK%TSNOW%TEMP(:,:) = DMK%XSNOWTEMP(:,:)
  PEK%TSNOW%TS  (:)   = DMK%XSNOWTEMP(:,1)
ENDIF
!

CALL UNPACK_DIAG_CPL_ESM_ISBA(IO, DEK, PK, PK%NR_P, PK%NSIZE_P, IO%NPATCH, JP,    &
                              ZCPL_DRAIN, ZCPL_RUNOFF, ZCPL_EFLOOD, ZCPL_PFLOOD,  &
                              ZCPL_IFLOOD, ZCPL_ICEFLUX, ZCPL_DOCFLUX             )  
!
!----------------------------------------------------------------------
!
! for further chemical biogenic emissions
!
IF (CHI%SVI%NBEQ>0 .AND. CHI%LCH_BIO_FLUX) THEN
  !
  DO JJ=1,PK%NSIZE_P
    ZSW_FORBIO(PK%NR_P(JJ),JP) = 0.
  ENDDO
  !
  DO JSWB=1,ISWB
!cdir nodep
!cdir unroll=8
    DO JJ=1,PK%NSIZE_P
      ZSW_FORBIO(PK%NR_P(JJ),JP) = ZSW_FORBIO(PK%NR_P(JJ),JP) + ZP_DIR_SW(JJ,JSWB) + ZP_SCA_SW(JJ,JSWB)  
    ENDDO
  ENDDO
  !
ENDIF
!----------------------------------------------------------------------
!
! Unpack output dummy arguments for each patch:
!
IF (IO%NPATCH==1) THEN
   ZSFTQ_TILE      (:,JP)  = ZP_SFTQ      (:)
   ZSFTH_TILE      (:,JP)  = ZP_SFTH      (:)
   ZSFTS_TILE      (:,:,JP)= ZP_SFTS      (:,:)
   ZSFCO2_TILE     (:,JP)  = ZP_SFCO2     (:)
   ZSFU_TILE       (:,JP)  = ZP_SFU       (:)
   ZSFV_TILE       (:,JP)  = ZP_SFV       (:)
   ZTRAD_TILE      (:,JP)  = ZP_TRAD      (:)
   ZTSURF_TILE     (:,JP)  = ZP_TSURF     (:)
   ZZ0_TILE        (:,JP)  = ZP_Z0        (:)
   ZZ0H_TILE       (:,JP)  = ZP_Z0H       (:)
   ZQSURF_TILE     (:,JP)  = ZP_QSURF     (:)   
ELSE
!cdir nodep
!cdir unroll=8
 DO JJ=1,PK%NSIZE_P
   JI = PK%NR_P(JJ)
   ZSFTQ_TILE      (JI,JP)  = ZP_SFTQ      (JJ)
   ZSFTH_TILE      (JI,JP)  = ZP_SFTH      (JJ)
   ZSFCO2_TILE     (JI,JP)  = ZP_SFCO2     (JJ)
   ZSFU_TILE       (JI,JP)  = ZP_SFU       (JJ)
   ZSFV_TILE       (JI,JP)  = ZP_SFV       (JJ)
   ZTRAD_TILE      (JI,JP)  = ZP_TRAD      (JJ)
   ZTSURF_TILE     (JI,JP)  = ZP_TSURF     (JJ)
   ZZ0_TILE        (JI,JP)  = ZP_Z0        (JJ)
   ZZ0H_TILE       (JI,JP)  = ZP_Z0H       (JJ)
   ZQSURF_TILE     (JI,JP)  = ZP_QSURF     (JJ)   
 ENDDO
!
!cdir nodep
!cdir unroll=8
  DO JK=1,SIZE(ZP_SFTS,2)
    DO JJ=1,PK%NSIZE_P
      JI=PK%NR_P(JJ)    
      ZSFTS_TILE      (JI,JK,JP)= ZP_SFTS      (JJ,JK)
    ENDDO
  ENDDO
ENDIF
!
!----------------------------------------------------------------------
!
! Get output dust flux if we are calculating dust
IF (NDSTMDE .GE. 1) IMOMENT = INT(IDST / NDSTMDE)
IF (CHI%SVI%NDSTEQ>0) THEN
  DO JSV = 1,NDSTMDE
    IF (IMOMENT == 1) THEN
      DSTK%XSFDST(:,JSV) = ZSFTS_TILE(:,NDST_MDEBEG+JSV-1,JP)
    ELSE
      DSTK%XSFDST(:,JSV) = ZSFTS_TILE(PK%NR_P,NDST_MDEBEG+(JSV-1)*IMOMENT+1,JP)
    END IF

    DSTK%XSFDSTM(:,JSV) = DSTK%XSFDSTM(:,JSV) + DSTK%XSFDST(:,JSV) * PTSTEP
  ENDDO
ENDIF
!
IF (LHOOK) CALL DR_HOOK('COUPLING_ISBA_n:TREAT_PATCH',1,ZHOOK_HANDLE)
!
END SUBROUTINE TREAT_PATCH
!==========================================================================================
END SUBROUTINE COUPLING_ISBA_n
